;;; system --- System library of FG42 -*- lexical-binding: t; -*-
;;
;; Copyright (c) 2010-2021 Sameer Rahmani & Contributors
;;
;; Author: Sameer Rahmani <lxsameer@gnu.org>
;; URL: https://gitlab.com/FG42/FG42
;; Version: 3.0.0
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;; Commentary:
;; `System' is just a state monad which holds the state of the editor.
;; Each system has to have a `start' function to start the setup process.
;;
;;; Code:
(defvar fg42/system '()
  "The default system of FG42.")


;;;###autoload
(defun fg42/system-start (system)
  "Start the given SYSTEM."
  (require 'fg42/utils)
  (require 'fg42/system/core)
  (require 'fg42/system/dependencies)
  (require 'fg42/system/cubes)

  (let* ((system-map (funcall system '())))
    ;; Weird right ? Well, I've tried the functional state passing style
    ;; but as it turns out the performance isn't great in elisp
    (setq fg42/system (fg42/system-install-dependencies system-map))
    (setq fg42/system (fg42/system-setup-cubes fg42/system))
    (setq fg42/system (fg42/system-setup-modes-hook fg42/system))

    (message "SYSTEM: %s"
             (fg42/system-get fg42/system :cubes))))


(provide 'fg42/system)
;;; system.el ends here
